# This short Python V2 & V3 demonstration program remotely controls a USB24 series board
# using serial commands to an FTDI serial port.
# Each of the 24 relays is activated sequentially.
# N. J. Halse V1.00  23/09/19

import serial    # serial inteface - you may need to use 'pip install pyserial' for this library
import time      # used for display
import sys       # display of data read back from the board.
import signal    # intercept SIGINT (Ctrl-C) to end demo.

# Define which FTDI port the USB24 device is using (Chage these according to your operating system and selected port mnumber.
SER_PORT_LIN = '/dev/ttyUSB0' # Linux version - check this using $> ls /dev/ttyUSB*
SER_PORT_WIN = 'COM29' # Windows version - check using device manager in control panel

shorttime = 0.01 # minimum time in seconds between commands (0.008 is the lowest reliable value)
longtime = 0.1 # time in seconds between commands that slows down this program (try changing it between 0.008 and 10 seconds)
outbanks = [b'\x43',b'\x46',b'\x4A'] # the 3 output banks. Note that 0x49 is not used
dirbanks = [b'\x42',b'\x45',b'\x48'] # tthe 3 direction control banks

# Introduction
print ("\nEasyDaq USB demonstration of the USB4 series boards")
print ("(Hit Ctrl-c to end)\n")
time.sleep(1)


# Open port
ser = serial.Serial(SER_PORT_WIN) #Open the port - select according to your OS. (Select WIN or LIN according to your operating system)
print (ser.name) # To check...

def bankn(bank):          # this function sequences through the 8 relaysw on the selected port
 B = [b'\x01',b'\x02',b'\x04',b'\x08',b'\x10',b'\x20',b'\x40',b'\x80']
 for y in B:
  MESSAGE = bank+y;       # send the message to control relay "y" on port "bank"
  ser.write(MESSAGE);
  time.sleep(longtime)
  MESSAGE = bank+b'\x00'; # clear all rerlays in the bank
  ser.write(MESSAGE);
  time.sleep(shorttime)

# Define a signal handler to catch when the user quits using Ctrl-C and then tidy up
def signal_handler(signal, frame):
 print('\n\n\n      *** Thank you for trying the EasyDaq USB4 series *** \nFor further information please visit http://www.easydaq.co.uk\n')
 time.sleep(shorttime)  # Just in case we were part way through sending a command pair of bytes.
 for z in outbanks:
  MESSAGE = z + b'\x00'  # relays to off
  ser.write(MESSAGE)
  time.sleep(shorttime)
 for z in dirbanks:
  MESSAGE = z+b'\xFF' # ports to input
  ser.write(MESSAGE)
  time.sleep(shorttime)
 ser.close()
 sys.exit(0)

signal.signal(signal.SIGINT, signal_handler) # Register the signal handler

# Configure the USB24 ports for all output
for z in dirbanks:
 MESSAGE = z + b'\x00'  # ports to output
 ser.write(MESSAGE)
 time.sleep(shorttime)

# The main loop.. Sequentially operating each relay in turn:
while True:
 for x in outbanks: # select a bank
  bankn(x);         # cycle he relays in that bank

